import { getRandomInt } from "../other/utilities.js";
import { addToStage, removeFromStage } from "./game.js";
export var ItemType;
(function (ItemType) {
    ItemType[ItemType["tail"] = 0] = "tail";
    ItemType[ItemType["food"] = 1] = "food";
    ItemType[ItemType["wall"] = 2] = "wall";
})(ItemType || (ItemType = {}));
export class Grid {
    constructor(args) {
        this.args = args;
        this.grid = [];
        for (let column = 0; column < args.columns; column++) {
            this.grid[column] = [];
            for (let line = 0; line < args.lines; line++) {
                this.grid[column][line] = [];
            }
        }
    }
    add(item, position) {
        const column = position.column;
        const line = position.line;
        if (column >= this.args.columns ||
            line >= this.args.lines ||
            column < 0 ||
            line < 0) {
            throw new Error("Invalid grid position.");
        }
        this.grid[column][line].push(item);
        const shape = item.shape;
        shape.x = column * Grid.size + Grid.halfSize;
        shape.y = line * Grid.size + Grid.halfSize;
        item.position = position;
        addToStage(shape);
    }
    remove(item) {
        const position = item.position;
        const index = this.grid[position.column][position.line].indexOf(item);
        if (index < 0) {
            throw Error('Item not in the correct position.');
        }
        this.grid[position.column][position.line].splice(index, 1);
        removeFromStage(item.shape);
        return item;
    }
    removeAll() {
        for (let column = 0; column < this.grid.length; column++) {
            const columns = this.grid[column];
            for (let line = 0; line < columns.length; line++) {
                const lines = columns[line];
                for (let a = lines.length - 1; a >= 0; a--) {
                    const item = lines[a];
                    this.remove(item);
                }
            }
        }
    }
    isEmpty(position) {
        return this.grid[position.column][position.line].length === 0;
    }
    getValidPosition(position) {
        let column = position.column;
        let line = position.line;
        const columnsLength = this.args.columns;
        const linesLength = this.args.lines;
        if (column < 0) {
            column = columnsLength - Math.abs(column);
        }
        else if (column >= columnsLength) {
            column = column - columnsLength;
        }
        if (line < 0) {
            line = linesLength - Math.abs(line);
        }
        else if (line >= linesLength) {
            line = line - linesLength;
        }
        return {
            column: column,
            line: line
        };
    }
    isValid(position) {
        if (position.column < 0 ||
            position.column >= this.args.columns ||
            position.line < 0 ||
            position.line >= this.args.lines) {
            return false;
        }
        return true;
    }
    move(item, to) {
        if (!item) {
            return;
        }
        const columns = this.args.columns;
        const lines = this.args.lines;
        if (to.column >= columns) {
            to.column = 0;
        }
        else if (to.column < 0) {
            to.column = columns - 1;
        }
        if (to.line >= lines) {
            to.line = 0;
        }
        else if (to.line < 0) {
            to.line = lines - 1;
        }
        const existingItems = this.grid[to.column][to.line];
        if (existingItems.length > 0) {
            for (let a = 0; a < existingItems.length; a++) {
                const otherItem = existingItems[a];
                this.args.onCollision({
                    a: item,
                    b: otherItem
                });
            }
        }
        const from = item.position;
        const index = this.grid[from.column][from.line].indexOf(item);
        if (index < 0) {
            throw Error('Item not in the correct position.');
        }
        this.grid[from.column][from.line].splice(index, 1);
        this.grid[to.column][to.line].push(item);
        item.shape.x = to.column * Grid.size + Grid.halfSize;
        item.shape.y = to.line * Grid.size + Grid.halfSize;
        item.position = to;
    }
    getRandomEmptyPosition(excludeRectangles) {
        const emptyPositions = [];
        const grid = this.grid;
        const columns = this.args.columns;
        const lines = this.args.lines;
        for (let column = 0; column < columns; column++) {
            for (let line = 0; line < lines; line++) {
                const items = grid[column][line];
                if (items.length === 0) {
                    const position = { column: column, line: line };
                    if (!positionInRectangles(position, excludeRectangles)) {
                        emptyPositions.push({
                            column: column,
                            line: line
                        });
                    }
                }
            }
        }
        if (emptyPositions.length === 0) {
            return {
                column: getRandomInt(0, columns),
                line: getRandomInt(0, lines)
            };
        }
        else {
            const index = getRandomInt(0, emptyPositions.length - 1);
            return emptyPositions[index];
        }
    }
}
Grid.size = 10;
Grid.halfSize = Grid.size / 2;
function positionInRectangles(position, rectangles) {
    if (!rectangles) {
        return false;
    }
    for (let a = 0; a < rectangles.length; a++) {
        const rect = rectangles[a];
        if (!(position.column < rect.position.column ||
            position.column > rect.position.column + rect.width ||
            position.line < rect.position.line ||
            position.line > rect.position.line + rect.height)) {
            return true;
        }
    }
}
